#/bin/sh -e

# Generate eCAP antivirus adapter sources from Factory templates in files/
# using user-provided generation parameters. Manual editing of generated
# sources is required to get a functional adapter.

configure()
{
    echo -n "Full Adapter name (e.g., eCAP BestAV Adapter): "
    read eCAP_AV_SDK_DESCRIPTION
    validateRequired "$eCAP_AV_SDK_DESCRIPTION"

    echo -n "Short Adapter name for logging (e.g., eBestAV): "
    read eCAP_AV_SDK_SHORT_NAME
    validateName "$eCAP_AV_SDK_SHORT_NAME"

    echo -n "Unique adapter library name (e.g., ecap_bestav_adapter): "
    read eCAP_AV_SDK_LIB_NAME
    validateName "$eCAP_AV_SDK_LIB_NAME"
    outDir="$eCAP_AV_SDK_LIB_NAME"
    mkdir "$outDir" || exit $?
    rmdir "$outDir"

    echo -n "C++ boilerplate file name (e.g., /tmp/company-boiler.txt): "
    read boilerFile
    validateRequired "$boilerFile"
    cat "$boilerFile" > /dev/null || exit $?
    eCAP_AV_SDK_BOILERPLATE=`cat "$boilerFile"` # may be empty

    echo -n "Unique service URI (e.g., ecap://mycompany.com/services/bestav): "
    read eCAP_AV_SDK_SERVICE_URI
    validateRequired "$eCAP_AV_SDK_SERVICE_URI"

    echo -n "Optional: 3rd-party libraries in LIBADD format (e.g., -lmyav): "
    read eCAP_AV_SDK_EXTERNAL_LIBADD # may be empty

    # We currently do not see enough need to make this user-configurable,
    # but our infrastructure is mostly ready for it.
    eCAP_AV_SDK_CLASS_NAME=MyAv

    # We currently do not see enough need to make these user-configurable,
    # but our infrastructure is ready for them.
    eCAP_AV_SDK_GUARD=`echo "$eCAP_AV_SDK_LIB_NAME" | tr '[:lower:]' '[:upper:]'`
    eCAP_AV_SDK_VERSION='0.0.0'
}

# $1 - required parameter value
validateRequired()
{
    if test -z "$1"
    then
        echo "Error: Got an empty value for the required parameter."
        exit 1
    fi
}

# $1 - value to validate
validateName()
{
    validateRequired "$1" # for now, all names are required
    if echo "$1" | grep -v "^[[:alnum:]_-]*$" > /dev/null
    then
        echo "Error: Most non-alphanumeric characters are prohibited: $1"
        exit 1
    fi
}

substituteMacro()
{
    name="$1"
    value=`eval echo '"$'$name'"'`
    #echo "$name=$value"
    find "$outDir" -type f | xargs sed -i "s@\${$name}@$value@g"
}

# TODO: Merge with substituteMacro()
substituteMultilineMacro()
{
    name="$1"
    rawValue=`eval echo '"$'$name'"'`
    # Our rawValue might have special RE characters (that we escape) and
    # probably has new lines (that we convert to \n).
    # This is further complicated by the backslash-eating eval.
    value=`echo "$rawValue" |
        awk '{ gsub(/[@&\\\\]/, "\\\\\\\\&"); printf("%s\\\\n", $0); }'`
    #echo "$name=$value"
    find "$outDir" -type f | xargs sed -i "s@\${$name}@$value@g"
}


echo "Creating a custom antivirus eCAP adapter using Factory SDK..."

if test \! -f ./files/src/MyAv.h
then
    echo "$0 must be run from the top-level SDK source directory"
    exit 1;
fi

configure

cp -pri files "$outDir"

substituteMultilineMacro eCAP_AV_SDK_BOILERPLATE
substituteMacro eCAP_AV_SDK_CLASS_NAME
substituteMacro eCAP_AV_SDK_DESCRIPTION
substituteMacro eCAP_AV_SDK_EXTERNAL_LIBADD
substituteMacro eCAP_AV_SDK_GUARD
substituteMacro eCAP_AV_SDK_LIB_NAME
substituteMacro eCAP_AV_SDK_SERVICE_URI
substituteMacro eCAP_AV_SDK_SHORT_NAME
substituteMacro eCAP_AV_SDK_VERSION


echo "Your $eCAP_AV_SDK_SHORT_NAME is ready in $outDir"
echo "Next steps are described in $outDir/todo.txt"

exit 0
